__version__ = '0.0.3-alpha'

# thanks tensorflow open source :D
# Special dunders that we choose to export:
_exported_dunders = set([
    '__version__',
    '__git_version__'
])

# This is necessary to export our dunders.
__all__ = [s for s in dir() if s in _exported_dunders or not s.startswith('_')]


import argparse
import os
import utils
import settings
import pandas as pd
import psycopg2
import yaml
log = utils.get_logger()

import sys
from sqlalchemy.orm import sessionmaker
from sqlalchemy.engine import create_engine
import logging.config

log = logging.getLogger(__name__)


def connection_string(engine, host, port, username, password, database):
    return '{engine}{username}:{password}@{host}:{port}/{database}'.format(
        engine=engine,
        username=username,
        password=password,
        host=host,
        port=port,
        database=database)


class DbSession(object):
    __instance = None

    def __new__(cls, *args, **kwargs):
        if DbSession.__instance is None:
            DbSession.__instance = object.__new__(cls)
            try:
                cls.engine = create_engine(connection_string(
                    engine=kwargs['engine'],
                    host=kwargs['host'],
                    port=kwargs['port'],
                    username=kwargs['username'],
                    password=kwargs['password'],
                    database=kwargs['database']
                ), echo=settings.DEBUG)
                log.debug('activating session')
                cls.session = sessionmaker(bind=cls.engine, autocommit=False, autoflush=True)()
            except KeyError as e:
                print('DbSession available')
        return DbSession.__instance

    def __init__(self, **kwargs):
        """
        :param username: a database username
        :param password: database password
        :param host: ip of host
        :param port: port of db
        :param database: a database name
        """
        pass


def write(con, df, table_name, if_exists='append'):
    try:
        df.to_sql(table_name, con=con, if_exists=if_exists)
    except (Exception, psycopg2.Error) as error:
        print("Could not write to table", error)

    return None


def populate_measure_type(con=None):
    df = pd.DataFrame({'measure_type_id': ['increase', 'decrease', 'neutral'], #1,2,3
                       'measure_type_description': ['increase in measure is good', 'decrease in measure is good', 'agnostic at the moment']}).set_index('measure_type_id')

    write(con=con, df=df, table_name='measure_type')
    log.info("--- Created measure types in measure type table ---")


def populate_perspectives(con=None):
    df = pd.DataFrame({'perspective_id': ('FINANCIAL', 
                                       'CUSTOMER',
                                       'INTERNAL_BUSINESS_PROCESSES',
                                       'EMPLOYEE_LEARNING_AND_GROWTH'),
                        'perspective_description': ('relating to the financial perspective', 'relating to the customer perspective', 
                        'relating to the internal and business processes perspective', 'relating to the development and growth of employees')
                        }).set_index('perspective_id')
    write(con=con, df=df, table_name='perspective')
    log.info("--- Created perspectives in perspective table ---")


def populate_entity_type(con=None):
    df = pd.DataFrame({'entity_type_id': ('grain_crops',
                                       'micro_green_crops',
                                       'green_crops',
                                       'small_livestock',
                                       'livestock',
                                       'resource_location'),
                       'entity_type_description': ('grain crop based farming',
                                                   'micro greens based farming',
                                                   'greens based farming (spinach etc.)',
                                                   'small livestock such as chickens',
                                                   'livestock such as goats or cows',
                                                   'Either physical or digital (location, bank account)')
                       }).set_index('entity_type_id')

    write(con=con, df=df, table_name='entity_type')
    log.info("--- Created entity types in entity_type table ---")

def populate_sensor_store(con=None):

    df = pd.DataFrame({'sensor_store_catalog_id': (1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18, 19,20,21,22,23,24,25,26),
                       'sensor_store_id': ('soil moisture','ambient temperature','water level',
                                            'weight (kg)','weight (mg)', 'humidity', 'soil temperature',
                                            'light intensity', 'water quality', 'soil NPK levels',
                                           'electricty usage (kwh)', 'power remaining','water rate usage (l/min)',
                                            'operating profit','Overhead cost', 'No of customers on database', 'No of loyal/returning customers',
                                            'Delivery rate', 'Return rate', 'financial books/till', 'No of employees', 'Product variation',
                                            'Mentor visit log','Customer tracker','Employee satisfaction tracker', 'New selling method tracker'),

                       'sensor_description': ('to measure soil moisture', 'air temperature', 'water level in a tank',
                                              'weight of something in kg', 'weight of something in mg', 'air humidity',
                                              'temperature of the soil', 'energy of the sunlight at point', 'dirt in water',
                                              'NPK ratio of soil', 'electricity usage', 'amount of electricity remaining',
                                              'rate of water usage in litres per min', 'The operating profit as indicated by the accounting books', 'The overhead cost as indicated by the books',
                                              'The number of customers on the database', 'The number of customers that are returning/loyal', 'The rate of delivery of products to customers',
                                               'The rate of return on products', 'Ledger or document of transactions and monetary assets', 'The number of people employed at the farm','The number of different products on sale',
                                               'The number of mentor visits required', 'Customer tracker','Employee satisfaction tracker','New selling method tracker'),
                       'sensor_active_indicator': (1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1)
                       }).set_index('sensor_store_catalog_id')
    write(con=con, df=df, table_name='sensor_store')
    log.info("--- Created base sensors sensor store ---")

def populate_objective_store(con=None):
    
    df = pd.DataFrame( {'objective_id': (
    'Gain acitivity earning',
    'Increase operational cash margin',
    'Improve production efficiency',
    'Increase total income', 
    'Improve profitability of investments',
    'Meet shareholder expectations',
    'Achieve profitable & sustainable growth',
    'Increase of the activities of the agricultural company',
     'Decrease of financial costs/ Reduce indirect expenditure',
     'Reduce operational costs',
     'Increase asset utilisation',
     'To develop demand', 
'Set up high level of customer satisfaction',
'To set up competitive prices',
'Better health status',
'Increase customer base',
'Customer loyalty',
'Image of the agricultural farm',
'Quality of produce',
'Meet customer demand',
'Improve product delivery',
'Increase customer network',
'Increase level of productivity ',
'Increase product quality',
'Improve milk yield',
'Improve utilisation of feed',
'Higher yield of forage ',
'Improved utilisation of milk quota',
'Less stoarge loss of forage',
'Improve utilisation of buildings',
'Improve utilisation of machinery',
'Reduce the use of labour',
'Development of strategic segments',
'Development of innovative processes',
'Observance of environmental laws',
'Soil processing and fertilising',
'Quality of plant cultivation',
'On time and in full products',
'Customer communication',
'Develop new selling methods',
'Create interdependency between the firm and the community',
'Increase product offering',
'Improve managerial/partner and technical competences',
'Provide employee motivation',
'Hiring of more skilled herd manager',
'Use of better information systems',
'Development of managerial competences',
'Develop safety protocols to protect staff',
'Implement practises that employ and retain best staff',
'Create organisation with strong and constructive culture',
'Parner/employee satisfaction'),

    'objective_description': ('Gain earnings on activities',
    'Gain earnings on activities',
     'Improve operational efficiencies and reduce wastes',
      'Increase total income',
    'Increase profitability of investments',
    'Satisfy the shareholder expecations','Achieve profitable & sustainable growth', 
    'Increase the number of activities of the farm', 
    'Decrease of financial costs/ Reduce indirect expenditure',
    'Reduce operational costs',
    'Increase asset utilisation',
    'To develop demand', 
'Set up high level of customer satisfaction',
'To set up competitive prices',
'Better health status',
'Increase customer base',
'Customer loyalty',
'Image of the agricultural farm',
'Quality of produce',
'Meet customer demand',
'Improve product delivery',
'Increase customer network',
'Increase level of productivity ',
'Increase product quality',
'Improve milk yield',
'Improve utilisation of feed',
'Higher yield of forage ',
'Improved utilisation of milk quota',
'Less stoarge loss of forage',
'Improve utilisation of buildings',
'Improve utilisation of machinery',
'Reduce the use of labour',
'Development of strategic segments',
'Development of innovative processes',
'Observance of environmental laws',
'Soil processing and fertilising',
'Quality of plant cultivation',
'On time and in full products',
'Customer communication',
'Develop new selling methods',
'Create interdependency between the firm and the community',
'Increase product offering',
'Improve managerial/partner and technical competences',
'Provide employee motivation',
'Hiring of more skilled herd manager',
'Use of better information systems',
'Development of managerial competences',
'Develop safety protocols to protect staff',
'Implement practises that employ and retain best staff',
'Create organisation with strong and constructive culture',
'Parner/employee satisfaction'),

    'perspective_id': ('FINANCIAL','FINANCIAL', 'FINANCIAL', 'FINANCIAL', 'FINANCIAL', 'FINANCIAL', 'FINANCIAL', 'FINANCIAL', 'FINANCIAL', 'FINANCIAL', 'FINANCIAL',
     'CUSTOMER', 'CUSTOMER', 'CUSTOMER', 'CUSTOMER', 'CUSTOMER', 'CUSTOMER', 'CUSTOMER', 'CUSTOMER', 'CUSTOMER', 'CUSTOMER', 'CUSTOMER',
     'INTERNAL_BUSINESS_PROCESSES','INTERNAL_BUSINESS_PROCESSES','INTERNAL_BUSINESS_PROCESSES','INTERNAL_BUSINESS_PROCESSES','INTERNAL_BUSINESS_PROCESSES','INTERNAL_BUSINESS_PROCESSES','INTERNAL_BUSINESS_PROCESSES','INTERNAL_BUSINESS_PROCESSES'
     ,'INTERNAL_BUSINESS_PROCESSES','INTERNAL_BUSINESS_PROCESSES','INTERNAL_BUSINESS_PROCESSES','INTERNAL_BUSINESS_PROCESSES','INTERNAL_BUSINESS_PROCESSES','INTERNAL_BUSINESS_PROCESSES','INTERNAL_BUSINESS_PROCESSES','INTERNAL_BUSINESS_PROCESSES'
     ,'INTERNAL_BUSINESS_PROCESSES','INTERNAL_BUSINESS_PROCESSES','INTERNAL_BUSINESS_PROCESSES','INTERNAL_BUSINESS_PROCESSES',
     'EMPLOYEE_LEARNING_AND_GROWTH','EMPLOYEE_LEARNING_AND_GROWTH','EMPLOYEE_LEARNING_AND_GROWTH','EMPLOYEE_LEARNING_AND_GROWTH','EMPLOYEE_LEARNING_AND_GROWTH','EMPLOYEE_LEARNING_AND_GROWTH','EMPLOYEE_LEARNING_AND_GROWTH','EMPLOYEE_LEARNING_AND_GROWTH'
     ,'EMPLOYEE_LEARNING_AND_GROWTH') }




    ).set_index('objective_id')
    write(con=con, df=df, table_name='objective_store')
    log.info("--- Created base objective store ---")


def populate_measure_store(con=None):

    df = pd.DataFrame({'measure_store_catalog_id': (
        1,2,3,4,5,
        6,7,8,9,10,
        11,12,13,14,15,
        16,17,18,19,20,
        21,22,23,24,25,
        26,27,28,29,30,
        31,32,33,34,35,
        36,37,38,39,40,
        41,42,43,44,45,
        46,47,48,49,50,
        51,52,53,54,55,
        56,57,58,59,60,
        61,62,63,64,65,
        66,67,68,69,70,
        71,72,73,74,75),

    'measure_store_id': (
        'Financial returns','Cash margin', 'Customer satisfaction', 'Percentage of product that hits target market', 'Number of active customers on database',
    'Volume/Weight of product sold', 'Cash received', 'Income statement', 'Cost drivers', 'Gross margin per unit of product sold', 
    'Revenue per employee','Overhead cost per product sold',  'Return on investment (ROI)', 'Invesment portfolio', 'Return on Assets (ROA)', 
    'Operating profit margin (OPM)', 'Economic value add (EVA)', 'Cost of capital (COC)', 'Increase turnover','Variation of operating profit', 
    'Variation of financial expenses','Indirect costs','Demand', 'Selling price','Selling price relative to competitors',
    'Customer loyalty', 'Product replacement rate', 'Number of required mentor visits', 'Percentage of new customers','Turnover from new customers',
    'Customer lost', 'Turnover by customer loyalty','Total rejected products to total revenue','Customer complaints to total customers', 'Demand satisfaction rate',
    'Delivery time','On time delivery','Number of sales proposals per month', 'Productivity','Perceived quality',
    'Crop/resource yield','Hygiene and sanitary standards','Animal condition score','Feed units per animal', 'Yeild of forage',
     'Crop rotation', 'Storage loss', 'Building utilisation','Equipment utilisation','Labour requirements', 
     'Strategic segment development','Number of varieties of products on the market','Number of new cultivation techniques per year', 'Seedling success rate','Environmental complaints',
     'Production success rate','Demand planning accuracy','Proportion of products defective','Customer offerings','Customer care calls',
     'Number of new selling methods per year','Sales from new selling methods','Number of community conflicts', 'Employee capacity','Improvement proposals',
     'Complaint resolution', 'Employee satisfaction','Education opportunities', 'Administration costs','Information availability',
     'The length of courses','Employee weekends off', 'Staff development','Health and safety breaches', 'Resource consumption'),


    'measure_description': (
        'The financial return on an activity', 'The margin of cash', 'How satisfied are the customers', 'The percentage of the target market reached','Number of active customers', 
    'A quantifiable measurement of a product sold', 'The amount of cash received', 'Evaluation of the income statement', 'Value of the cost drivers',  'The profit margin per unit of of product',
     'The revenue earned divided by the number of employees', 'The cost to produce one unit of product', 'The return on various investements', 'The diversity of the investment portfolio', 'The return divided by the value of all the assets', 
     'Profit on sales after paying for variable costs of production','The value of the product minus the input costs', 'The cost of acquiring capital (investments etc.)', 'Increase the amount of money turned over','Diversify the avenues of operating profit',
     'The variety of financial expenses','The value of indirect costs','The volume/quantity of customer demand', 'The price at which products are sold', 'The price at which products are sold relative to competitors',
     'The number of return customers as a percentage of total customers', 'The number of times customers request product exchanges as a percentage of total sales', 'The number of times a mentor is required to visit a farm','Percentage or number of new customers', 'Turnover/sales to new customers',
     'The number/percentage of customers who do not return','The value of the turnover by loyal/return customers','The value of the rejected products compared to total products sold','The number of complaints divided by the number of customers','Unfulfilled demand to total demand',
     'The average delivery time of products', 'Proportion of deliveries delivered on time','Number of new sales avenues', 'The overall productivity (Time spent per product produced)', 'The perceived quality by the customers',
     'Increase the yield on the products per resource (Increase crop yield per plot)','The number of hygiene and sanitary standards and processes', 'A score out of 10 on animal wellbeing and health','Amount of feed units per animal','The yield per unit of forage used',
     'Crop rotation', 'Losses due to inadequate storage', 'The utilisation of the buildings (Time spent in use/total time)', 'The amount of time equipment is used/total time','The amount of manual labour required',
     'Number of products with strategic value', 'Number of different products on the market','The number of new culitvation techniques tried per year','The number of seedlings succeeded/total planted', 'Number of environmental related complaints',
    'Successful crop production per square meter', 'The accuracy of the demand planning', 'The proportion of products that are defective', 'The number of offers made to customers per year', 'Calls to customer care per year',
     'Total number of new selling methods/avenues per year','Proportion of sales from new selling methods', 'The number of community complaints/incidents', 'The management capacity of new employees', 'The number of improvement proposals identified',
     'Number of complaints resolved/total complaints', 'The level of employee satisfaction out of 10', 'The number of education opportunities for employees','The costs of administration and bookkeeping', 'The number of different information streams',
     'The time spent/length of education courses','Number of rest/off days', 'Time spent on staff development', 'Number of health and safety related breaches', 'Rate of resource consumption'),


    'measure_type_id': (
        'increase','increase','increase','increase','increase',
        'increase','increase','increase','increase','increase',
        'increase','decrease','increase','increase','increase',
        'increase','increase','decrease','increase','increase',
        'decrease','decrease','increase','neutral','decrease',
        'increase','decrease','decrease','increase','increase',
        'decrease','increase','decrease','decrease','decrease',
        'decrease','increase','increase','decrease','increase',
        'increase','increase','increase','decrease','increase',
        'increase','decrease','increase','increase','decrease',
        'increase','increase','increase','increase','decrease',
        'increase','increase','decrease','increase','decrease',
        'increase','increase','decrease','increase','increase',
        'increase','increase','increase','decrease','increase',
        'neutral','increase','increase','decrease','decrease')


}

    ).set_index('measure_store_catalog_id')
    write(con=con, df=df, table_name='measure_store')
    log.info("--- Created base measure store ---")


def populate_measure_sensor(con=None):
    df = pd.DataFrame (
    { 'measure_sensor_relationship_id': (
        1,2,3,4,5,
        6,7,8,9,10,
        11,12,13,14,15,
        16,17,18,19,20,
        21,22,23,24,25,
        26,27,28,29,30,
        31,32,33,34,35,
        36,37,38,39,40,
        41,42,43,44,45,
        46,47,48,49,50,
        51,52,53,54,55,
        56,57,58,59,60,
        61,62,63,64,65,
        66,67,68,69,70,
        71,72,73,74,75,
        76,77,78,79,80,
        81,82,83,84,85,
        86,87,88,89,90,
        91,92,93,94,95,
        96,97,98,99,100
        ,101,102,103,104,
        105,106,107,108,
        109,110,111,112,
        113,114,115,116,
        117,118,119,120,
        121,122,123,124,
        125,126,127,128,
        129,130,131,132,
        133,134,135,136,
        137,138,139,140,
        141,142,143,144,
        145,146,147,148,
        149,150,151,152,
        153,154,155,156,
        157,158,159),
    'measure_store_catalog_id': (1,1,1,1,2,2,2,3,3,3,4,4,5,5,6,6,7,7,7,8,8,8,8,9,9,9,9,9,10,10,10,11,11,11,11,12,13,13,15,15,15,16,16,16,17,17,17,18,18,18,20,20,20,21,21,21,22,22,22,23,23,23,23,24,24,25,25,26,26,26,27,27,27,27,28,29,30,30,30,31,31,31,32,32,32,32,33,33,34,34,34,35,39,39,39,40,41,41,41,41,41,41,41,41,41,41,41,41,45,45,45,45,45,45,45,45,45,45,45,47,48,48,50,50,50,52,54,54,54,54,54,54,54,54,54,54,54,54,54,55,58,60,61,63,67,69,75,75,75,75,75,75,75,75,75,75,75,75,75),
    'measure_store_id': ('Financial returns','Financial returns','Financial returns','Financial returns','Cash margin',
    'Cash margin','Cash margin','Customer satisfaction','Customer satisfaction',
    'Customer satisfaction','Percentage of product that hits target market','Percentage of product that hits target market','Number of active customers on database','Number of active customers on database',
    'Volume/Weight of product sold','Volume/Weight of product sold','Cash received','Cash received','Cash received',
    'Income statement','Income statement','Income statement','Income statement',
    'Cost drivers','Cost drivers','Cost drivers','Cost drivers','Cost drivers',
    'Gross margin per unit of product sold','Gross margin per unit of product sold','Gross margin per unit of product sold','Revenue per employee','Revenue per employee',
    'Revenue per employee','Revenue per employee','Overhead cost per product sold','Return on investment (ROI)','Return on investment (ROI)',
    'Return on Assets (ROA)','Return on Assets (ROA)','Return on Assets (ROA)','Operating profit margin (OPM)','Operating profit margin (OPM)',
    'Operating profit margin (OPM)','Economic value add (EVA)','Economic value add (EVA)','Economic value add (EVA)','Cost of capital (COC)',
    'Cost of capital (COC)','Cost of capital (COC)','Variation of operating profit','Variation of operating profit','Variation of operating profit','Variation of financial expenses',
    'Variation of financial expenses','Variation of financial expenses','Indirect costs','Indirect costs','Indirect costs',
    'Demand','Demand','Demand','Demand','Selling price','Selling price',
    'Selling price relative to competitors','Selling price relative to competitors','Customer loyalty','Customer loyalty','Customer loyalty',
    'Product replacement rate','Product replacement rate','Product replacement rate','Product replacement rate','Number of required mentor visits',
    'Percentage of new customers','Turnover from new customers','Turnover from new customers','Turnover from new customers','Customer lost',
    'Customer lost','Customer lost','Turnover by customer loyalty','Turnover by customer loyalty','Turnover by customer loyalty',
    'Turnover by customer loyalty','Total rejected products to total revenue','Total rejected products to total revenue','Customer complaints to total customers',
    'Customer complaints to total customers','Customer complaints to total customers','Demand satisfaction rate','Productivity','Productivity',
    'Productivity','Perceived quality','Crop/resource yield','Crop/resource yield','Crop/resource yield',
    'Crop/resource yield','Crop/resource yield','Crop/resource yield','Crop/resource yield','Crop/resource yield',
    'Crop/resource yield','Crop/resource yield','Crop/resource yield','Crop/resource yield',
    'Yeild of forage','Yeild of forage','Yeild of forage','Yeild of forage','Yeild of forage',
    'Yeild of forage','Yeild of forage','Yeild of forage','Yeild of forage',
    'Yeild of forage','Yeild of forage','Storage loss','Building utilisation',
    'Building utilisation','Labour requirements','Labour requirements','Labour requirements',
    'Number of varieties of products on the market','Seedling success rate','Seedling success rate','Seedling success rate',
    'Seedling success rate','Seedling success rate','Seedling success rate','Seedling success rate',
    'Seedling success rate','Seedling success rate','Seedling success rate','Seedling success rate',
    'Seedling success rate','Seedling success rate','Environmental complaints','Proportion of products defective',
    'Customer care calls','Number of new selling methods per year','Number of community conflicts','Employee satisfaction','Administration costs',
    'Resource consumption','Resource consumption','Resource consumption','Resource consumption','Resource consumption',
    'Resource consumption','Resource consumption','Resource consumption','Resource consumption','Resource consumption',
    'Resource consumption','Resource consumption','Resource consumption'),
    'sensor_store_catalog_id': (
        14,15,20,22,14,
        15,20,16,17,19,
        16,17,16,17,4,
        5,14,15,20,14,
        15,17,20,10,11,
        13,14,15,4,14,
        15,14,15,20,21,
        15,15,20,14,15,
        20,14,15,20,14,
        15,20,14,15,20,
        14,20,22,14,15,
        20,14,15,20,14,
        16,17,20,14,20,
        14,20,16,17,19,
        15,18,19,20,23,
        24,14,20,24,16,
        17,24,14,16,17,
        24,19,20,16,17,
        24,24,14,15,21,
        24,1,2,3,4,5,6,
        7,8,9,10,11,13,
        1,2,3,4,5,6,7,
        8,9,10,11,15,
        14,15,15,21,23,
        22,1,2,3,4,5,
        6,7,8,9,10,11,
        13,22,24,19,24,
        26,24,25,15,1,2,
        3,4,5,6,7,8,9,
        10,11,12,13),
    'sensor_store_id':( 'operating profit', 'Overhead cost', 'financial books/till', 'Product variation', 'operating profit', 'Overhead cost', 'financial books/till', 'No of customers on database', 'No of loyal/returning customers', 'Return rate', 'No of customers on database', 'No of loyal/returning customers', 'No of customers on database', 'No of loyal/returning customers', 'weight (kg)', 'weight (mg)', 'operating profit', 'Overhead cost', 'financial books/till', 'operating profit', 'Overhead cost', 'No of loyal/returning customers', 'financial books/till', 'soil NPK levels', 'electricty usage (kwh)', 'water rate usage (l/min)', 'operating profit', 'Overhead cost', 'weight (kg)', 'operating profit', 'Overhead cost', 'operating profit', 'Overhead cost', 'financial books/till', 'No of employees', 'Overhead cost', 'Overhead cost', 'financial books/till', 'operating profit', 'Overhead cost', 'financial books/till', 'operating profit', 'Overhead cost', 'financial books/till', 'operating profit', 'Overhead cost', 'financial books/till', 'operating profit', 'Overhead cost', 'financial books/till', 'operating profit', 'financial books/till', 'Product variation', 'operating profit', 'Overhead cost', 'financial books/till', 'operating profit', 'Overhead cost', 'financial books/till', 'operating profit', 'No of customers on database', 'No of loyal/returning customers', 'financial books/till', 'operating profit', 'financial books/till', 'operating profit', 'financial books/till', 'No of customers on database', 'No of loyal/returning customers', 'Return rate', 'Overhead cost', 'Delivery rate', 'Return rate', 'financial books/till', 'Mentor visit log', 'Customer tracker', 'operating profit', 'financial books/till', 'Customer tracker', 'No of customers on database', 'No of loyal/returning customers', 'Customer tracker', 'operating profit', 'No of customers on database', 'No of loyal/returning customers', 'Customer tracker', 'Return rate', 'financial books/till', 'No of customers on database', 'No of loyal/returning customers', 'Customer tracker', 'Customer tracker', 'operating profit', 'Overhead cost', 'No of employees', 'Customer tracker', 'soil moisture', 'ambient temperature', 'water level', 'weight (kg)', 'weight (mg)', 'humidity', 'soil temperature', 'light intensity', 'water quality', 'soil NPK levels', 'electricty usage (kwh)', 'water rate usage (l/min)', 'soil moisture', 'ambient temperature', 'water level', 'weight (kg)', 'weight (mg)', 'humidity', 'soil temperature', 'light intensity', 'water quality', 'soil NPK levels', 'electricty usage (kwh)', 'Overhead cost', 'operating profit', 'Overhead cost', 'Overhead cost', 'No of employees', 'Mentor visit log', 'Product variation', 'soil moisture', 'ambient temperature', 'water level', 'weight (kg)', 'weight (mg)', 'humidity', 'soil temperature', 'light intensity', 'water quality', 'soil NPK levels', 'electricty usage (kwh)', 'water rate usage (l/min)', 'Product variation', 'Customer tracker', 'Return rate', 'Customer tracker', 'New selling method tracker', 'Customer tracker', 'Employee satisfaction tracker', 'Overhead cost', 'soil moisture', 'ambient temperature', 'water level', 'weight (kg)', 'weight (mg)', 'humidity', 'soil temperature', 'light intensity', 'water quality', 'soil NPK levels', 'electricty usage (kwh)', 'power remaining', 'water rate usage (l/min)')
    }
    ).set_index('measure_sensor_relationship_id')
    write(con=con, df=df, table_name='measure_sensor')
    log.info("--- Created base measure-sensor relationship ---")

def populate_objective_measure(con=None):


   df=pd.DataFrame({ 'objective_measure_relationship_id': (1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,
      21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,
      41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,
      61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,
      81,82,83,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,
      101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,
      121,122,123,124,125,126,127,128,129,130,131,132,133,134,135,136,137,138,139,140,
      141,142,143,144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,160,
      161,162,163,164,165,166,167,168,169,170,171,172,173,174,175,176,177,178,179,180,
      181,182,183,184,185,186,187,188,189,190,191,192,193,194,195,196,197,198,199,200,
      201,202,203,204,205,206,207,208,209,210,211,212,213,214,215,216,217,218,219,220,
      221,222,223,224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239,240,
      241,242,243,244,245,246,247,248,249,250,251,252,253,254,255,256,257,258,259,260,
      261,262,263,264,265,266,267,268,269,270,271,272,273,274,275,276,277,278,279,280,
      281,282,283,284,285,286,287,288,289,290,291,292,293,294,295,296,297,298,299,300,
      301,302,303,304,305,306,307,308,309,310,311,312,313,314,315,316,317,318,319,320,
      321,322,323,324,325,326,327,328,329,330,331,332,333,334,335,336,337,338,339,340,
      341,342,343,344,345,346,347,348,349,350,351,352,353,354,355,356,357,358,359,360,
      361,362,363,364,365,366,367,368,369,370,371,372,373,374,375,376,377,378,379,380,
      381,382,383,384,385,386,387,388,389,390,391,392,393,394,395,396,397,398,399,400,
      401,402,403,404,405,406,407,408,409,410,411,412,413,414,415,416,417,418,419,420,
      421,422,423,424,425,426,427,428,429,430,431,432,433,434,435,436,437,438,439,440,
      441,442,443,444,445,446,447,448,449,450,451,452,453,454,455,456,457,458,459,
      460,461,462,463,464,465),
      
      'objective_id': ( 
 'Gain acitivity earning', 'Gain acitivity earning', 'Gain acitivity earning', 'Gain acitivity earning', 'Gain acitivity earning', 'Gain acitivity earning', 'Gain acitivity earning', 'Increase operational cash margin', 'Increase operational cash margin',
  'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin', 
 'Increase operational cash margin', 'Increase operational cash margin', 
 'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin',
  'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin', 
  'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin', 
  'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin', 
  'Increase operational cash margin', 'Increase operational cash margin', 'Increase operational cash margin', 'Improve production efficiency', 'Improve production efficiency', 'Improve production efficiency', 'Improve production efficiency', 'Improve production efficiency', 
  'Improve production efficiency', 'Improve production efficiency', 'Improve production efficiency', 'Improve production efficiency', 'Improve production efficiency', 'Improve production efficiency', 'Improve production efficiency', 'Improve production efficiency',
   'Improve production efficiency', 'Improve production efficiency', 'Improve production efficiency', 'Improve production efficiency', 'Improve production efficiency', 'Improve production efficiency', 'Improve production efficiency', 'Improve production efficiency', 
   'Improve production efficiency', 'Improve production efficiency', 'Improve production efficiency', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income',
    'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 
    'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 
    'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income', 'Increase total income',
     'Increase total income', 'Increase total income', 'Increase total income', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth',
      'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 
      'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth',
       'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth',
        'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 
        'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 
        'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 
        'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 
        'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth',
         'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth',
          'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 
          'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth',
           'Achieve profitable & sustainable growth', 'Achieve profitable & sustainable growth', 'Decrease of financial costs/ Reduce indirect expenditure', 'Decrease of financial costs/ Reduce indirect expenditure', 'Reduce operational costs', 'Reduce operational costs',
            'Reduce operational costs', 'Reduce operational costs', 'Reduce operational costs', 'Reduce operational costs', 'Reduce operational costs', 'Reduce operational costs', 'Reduce operational costs', 'Reduce operational costs', 'Reduce operational costs',
             'Reduce operational costs', 'Reduce operational costs', 'Reduce operational costs', 'Set up high level of customer satisfaction', 'Set up high level of customer satisfaction', 'Set up high level of customer satisfaction', 'Set up high level of customer satisfaction',
              'Set up high level of customer satisfaction', 'Set up high level of customer satisfaction', 'Set up high level of customer satisfaction', 'Set up high level of customer satisfaction', 'Set up high level of customer satisfaction', 'Set up high level of customer satisfaction',
               'Set up high level of customer satisfaction', 'Set up high level of customer satisfaction', 'Set up high level of customer satisfaction', 'Set up high level of customer satisfaction', 'Set up high level of customer satisfaction', 
               'Set up high level of customer satisfaction', 'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices',
                'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices',
                 'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices', 
                 'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices', 'To set up competitive prices', 'Increase customer base',
                  'Increase customer base', 'Increase customer base', 'Increase customer base', 'Increase customer base', 'Increase customer base', 'Increase customer base', 'Increase customer base', 'Increase customer base', 'Increase customer base', 'Customer loyalty', 'Customer loyalty', 
                  'Customer loyalty', 'Customer loyalty', 'Customer loyalty', 'Customer loyalty', 'Image of the agricultural farm', 'Image of the agricultural farm', 'Image of the agricultural farm', 'Image of the agricultural farm', 'Image of the agricultural farm', 'Image of the agricultural farm',
                   'Image of the agricultural farm', 'Image of the agricultural farm', 'Image of the agricultural farm', 'Image of the agricultural farm', 'Quality of produce', 'Quality of produce', 'Meet customer demand', 'Meet customer demand', 'Meet customer demand', 'Meet customer demand',
                    'Meet customer demand', 'Meet customer demand', 'Meet customer demand', 'Meet customer demand', 'Meet customer demand', 'Improve product delivery', 'Improve product delivery', 'Increase customer network', 'Increase customer network', 'Increase customer network', 
                    'Increase customer network', 'Increase customer network', 'Increase customer network', 'Increase level of productivity ', 'Increase level of productivity ', 'Increase level of productivity ', 'Increase level of productivity ', 'Increase level of productivity ',
                     'Increase level of productivity ', 'Increase level of productivity ', 'Increase level of productivity ', 'Increase level of productivity ', 'Increase level of productivity ', 'Increase level of productivity ', 'Increase level of productivity ', 
                     'Increase level of productivity ', 'Increase level of productivity ', 'Increase level of productivity ', 'Increase level of productivity ', 'Increase level of productivity ', 'Increase level of productivity ', 'Increase product quality', 'Increase product quality',
                      'Increase product quality', 'Increase product quality', 'Increase product quality', 'Increase product quality', 'Improve utilisation of feed', 'Higher yield of forage ', 'Higher yield of forage ', 'Less stoarge loss of forage', 'Improve utilisation of buildings', 'Improve utilisation of machinery', 
                      'Improve utilisation of machinery', 'Improve utilisation of machinery', 'Improve utilisation of machinery', 'Improve utilisation of machinery', 'Improve utilisation of machinery', 'Improve utilisation of machinery', 'Improve utilisation of machinery',
                       'Improve utilisation of machinery', 'Improve utilisation of machinery', 'Reduce the use of labour', 'Reduce the use of labour', 'Reduce the use of labour', 'Reduce the use of labour', 'Reduce the use of labour', 'Reduce the use of labour',
                        'Reduce the use of labour', 'Reduce the use of labour', 'Reduce the use of labour', 'Reduce the use of labour', 'Reduce the use of labour', 'Reduce the use of labour', 'Reduce the use of labour', 'Reduce the use of labour', 'Reduce the use of labour', 'Development of strategic segments', 
                        'Development of strategic segments', 'Development of strategic segments', 'Development of strategic segments', 'Development of innovative processes', 'Development of innovative processes', 'Development of innovative processes', 'Development of innovative processes', 
                        'Development of innovative processes', 'Development of innovative processes', 'Development of innovative processes', 'Development of innovative processes', 'Development of innovative processes', 'Observance of environmental laws', 'Observance of environmental laws', 
                        'Observance of environmental laws', 'Observance of environmental laws', 'Soil processing and fertilising', 'Soil processing and fertilising', 'Soil processing and fertilising', 'Soil processing and fertilising', 'Soil processing and fertilising', 
                        'Soil processing and fertilising', 'Soil processing and fertilising', 'Soil processing and fertilising', 'Soil processing and fertilising', 'Soil processing and fertilising', 'Soil processing and fertilising', 'Soil processing and fertilising', 
                        'Soil processing and fertilising', 'Soil processing and fertilising', 'Quality of plant cultivation', 'Quality of plant cultivation', 'Quality of plant cultivation', 'Quality of plant cultivation', 'Quality of plant cultivation', 'Quality of plant cultivation',
                         'Quality of plant cultivation', 'Quality of plant cultivation', 'Quality of plant cultivation', 'Quality of plant cultivation', 'Quality of plant cultivation', 'Quality of plant cultivation', 'On time and in full products', 'On time and in full products', 
                         'On time and in full products', 'On time and in full products', 'On time and in full products', 'On time and in full products', 'On time and in full products', 'On time and in full products', 'On time and in full products', 'Customer communication', 'Customer communication', 
                         'Customer communication', 'Develop new selling methods', 'Develop new selling methods', 'Develop new selling methods', 'Create interdependency between the firm and the community', 'Create interdependency between the firm and the community', 'Create interdependency between the firm and the community',
                          'Create interdependency between the firm and the community', 'Create interdependency between the firm and the community', 'Create interdependency between the firm and the community', 'Create interdependency between the firm and the community', 'Create interdependency between the firm and the community',
                           'Create interdependency between the firm and the community', 'Create interdependency between the firm and the community', 'Create interdependency between the firm and the community', 'Create interdependency between the firm and the community', 
                           'Create interdependency between the firm and the community', 'Increase product offering', 'Increase product offering', 'Increase product offering', 'Increase product offering', 'Increase product offering', 'Increase product offering', 'Increase product offering', 
                           'Increase product offering', 'Increase product offering', 'Improve managerial/partner and technical competences', 'Improve managerial/partner and technical competences', 'Improve managerial/partner and technical competences', 
                           'Improve managerial/partner and technical competences', 'Provide employee motivation', 'Provide employee motivation', 'Provide employee motivation', 'Provide employee motivation', 'Provide employee motivation', 'Use of better information systems', 'Development of managerial competences',
                            'Development of managerial competences', 'Development of managerial competences', 'Develop safety protocols to protect staff', 'Develop safety protocols to protect staff', 'Develop safety protocols to protect staff', 'Develop safety protocols to protect staff', 'Implement practises that employ and retain best staff',
                             'Implement practises that employ and retain best staff', 'Implement practises that employ and retain best staff', 'Implement practises that employ and retain best staff', 'Create organisation with strong and constructive culture', 'Create organisation with strong and constructive culture', 'Create organisation with strong and constructive culture',
                              'Create organisation with strong and constructive culture', 'Create organisation with strong and constructive culture', 'Parner/employee satisfaction', 'Parner/employee satisfaction', 'Parner/employee satisfaction', 'Parner/employee satisfaction', 'Parner/employee satisfaction', 'Parner/employee satisfaction', 
                              'Parner/employee satisfaction', 'Parner/employee satisfaction', 'Parner/employee satisfaction', 'Parner/employee satisfaction', 'Parner/employee satisfaction', 'Parner/employee satisfaction', 'Parner/employee satisfaction', 'Parner/employee satisfaction', 'Parner/employee satisfaction',
                               'Parner/employee satisfaction'
      ),

      'measure_store_catalog_id': (1,2,8,19,20,21,29,1,2,6,7,8,9,10,11,12,13,15,16,17,18,19,22,23,24,27,33,39,41,44,45,46,47,48,49,50,51,53,54,57,58,61,65,69,75,20,22,27,7,29,40,41,42,43,44,45,46,47,48,49,50,54,57,58,61,65,68,70,75,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,19,22,24,25,26,27,29,39,41,44,45,46,47,48,49,50,51,54,56,57,58,64,69,70,75,1,2,3,4,5,6,7,9,10,11,12,13,14,15,16,17,18,20,21,22,23,24,25,26,27,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,10,12,12,22,41,42,43,45,46,47,48,49,50,51,53,69,25,27,33,34,35,36,37,42,52,55,58,59,60,63,66,70,4,6,7,12,17,18,20,21,22,24,25,27,39,41,43,44,45,46,47,48,49,50,51,53,54,56,57,58,65,69,24,25,31,51,52,58,59,60,63,66,3,5,26,31,32,34,3,26,28,34,40,42,55,60,66,74,3,40,23,35,36,37,40,41,42,57,58,36,37,24,25,59,60,63,66,1,2,3,4,6,11,27,39,41,44,45,46,49,50,54,56,58,75,3,6,26,40,41,46,44,44,45,47,48,1,6,9,13,15,18,19,22,39,49,11,22,39,41,42,44,45,46,49,50,53,65,67,68,73,51,61,65,68,51,53,61,65,68,70,71,73,75,55,63,66,70,6,17,19,22,39,40,41,43,45,46,50,54,56,75,3,4,6,26,27,32,33,34,40,52,53,54,23,35,36,37,39,41,56,57,58,60,66,70,61,62,70,26,29,31,32,33,34,35,55,59,60,63,66,67,38,41,45,46,51,52,53,61,62,70,71,72,73,67,68,70,72,73,70,70,71,73,70,71,73,74,70,71,73,74,70,71,72,73,74,1,2,5,6,7,8,11,13,14,15,16,17,70,72,73,74),
            
            'measure_store_id': (  'Financial returns', 'Cash margin', 'Income statement', 'Increase turnover', 'Variation of operating profit', 'Variation of financial expenses', 'Percentage of new customers', 'Financial returns', 'Cash margin', 'Volume/Weight of product sold', 'Cash received', 'Income statement', 
            'Cost drivers', 'Gross margin per unit of product sold', 'Revenue per employee', 'Overhead cost per product sold', 'Return on investment (ROI)', 'Return on Assets (ROA)', 'Operating profit margin (OPM)', 'Economic value add (EVA)', 'Cost of capital (COC)', 'Increase turnover', 'Indirect costs', 'Demand', 
            'Selling price', 'Product replacement rate', 'Total rejected products to total revenue', 'Productivity', 'Crop/resource yield', 'Feed units per animal', 'Yeild of forage', 'Crop rotation', 'Storage loss', 'Building utilisation', 'Equipment utilisation', 'Labour requirements', 'Strategic segment development', 
            'Number of new cultivation techniques per year', 'Seedling success rate', 'Demand planning accuracy', 'Proportion of products defective', 'Number of new selling methods per year', 'Improvement proposals', 'Administration costs', 'Resource consumption', 'Variation of operating profit', 'Indirect costs', 'Product replacement rate',
             'Cash received', 'Percentage of new customers', 'Perceived quality', 'Crop/resource yield', 'Hygiene and sanitary standards', 'Animal condition score', 'Feed units per animal', 'Yeild of forage', 'Crop rotation', 'Storage loss', 'Building utilisation', 'Equipment utilisation', 'Labour requirements', 'Seedling success rate', 
             'Demand planning accuracy', 'Proportion of products defective', 'Number of new selling methods per year', 'Improvement proposals', 'Education opportunities', 'Information availability', 'Resource consumption', 'Financial returns', 'Cash margin', 'Customer satisfaction', 'Percentage of product that hits target market', 
             'Number of active customers on database', 'Volume/Weight of product sold', 'Cash received', 'Income statement', 'Cost drivers', 'Gross margin per unit of product sold', 'Revenue per employee', 'Overhead cost per product sold', 'Return on investment (ROI)', 'Invesment portfolio', 'Return on Assets (ROA)', 'Operating profit margin (OPM)', 
             'Increase turnover', 'Indirect costs', 'Selling price', 'Selling price relative to competitors', 'Customer loyalty', 'Product replacement rate', 'Percentage of new customers', 'Productivity', 'Crop/resource yield', 'Feed units per animal', 'Yeild of forage', 'Crop rotation', 'Storage loss', 'Building utilisation', 'Equipment utilisation',
              'Labour requirements', 'Strategic segment development', 'Seedling success rate', 'Production success rate', 'Demand planning accuracy', 'Proportion of products defective', 'Employee capacity', 'Administration costs', 'Information availability', 'Resource consumption', 'Financial returns', 'Cash margin', 'Customer satisfaction', 
              'Percentage of product that hits target market', 'Number of active customers on database', 'Volume/Weight of product sold', 'Cash received', 'Cost drivers', 'Gross margin per unit of product sold', 'Revenue per employee', 'Overhead cost per product sold', 'Return on investment (ROI)', 'Invesment portfolio', 'Return on Assets (ROA)',
               'Operating profit margin (OPM)', 'Economic value add (EVA)', 'Cost of capital (COC)', 'Variation of operating profit', 'Variation of financial expenses', 'Indirect costs', 'Demand', 'Selling price', 'Selling price relative to competitors', 'Customer loyalty', 'Product replacement rate', 'Percentage of new customers', 
               'Turnover from new customers', 'Customer lost', 'Turnover by customer loyalty', 'Total rejected products to total revenue', 'Customer complaints to total customers', 'Demand satisfaction rate', 'Delivery time', 'On time delivery', 'Number of sales proposals per month', 'Productivity', 'Perceived quality', 'Crop/resource yield',
                'Hygiene and sanitary standards', 'Animal condition score', 'Feed units per animal', 'Yeild of forage', 'Crop rotation', 'Storage loss', 'Building utilisation', 'Equipment utilisation', 'Labour requirements', 'Strategic segment development', 'Number of varieties of products on the market', 'Number of new cultivation techniques per year',
                 'Seedling success rate', 'Environmental complaints', 'Production success rate', 'Demand planning accuracy', 'Proportion of products defective', 'Customer offerings', 'Customer care calls', 'Number of new selling methods per year', 'Sales from new selling methods', 'Number of community conflicts', 'Employee capacity', 'Improvement proposals', 
                 'Complaint resolution', 'Employee satisfaction', 'Education opportunities', 'Administration costs', 'Information availability', 'The length of courses', 'Employee weekends off', 'Staff development', 'Health and safety breaches', 'Resource consumption', 'Gross margin per unit of product sold', 'Overhead cost per product sold', 'Overhead cost per product sold', 
                 'Indirect costs', 'Crop/resource yield', 'Hygiene and sanitary standards', 'Animal condition score', 'Yeild of forage', 'Crop rotation', 'Storage loss', 'Building utilisation', 'Equipment utilisation', 'Labour requirements', 'Strategic segment development', 'Number of new cultivation techniques per year', 'Administration costs', 
                 'Selling price relative to competitors', 'Product replacement rate', 'Total rejected products to total revenue', 'Customer complaints to total customers', 'Demand satisfaction rate', 'Delivery time', 'On time delivery', 'Hygiene and sanitary standards', 'Number of varieties of products on the market', 'Environmental complaints',
                  'Proportion of products defective', 'Customer offerings', 'Customer care calls', 'Number of community conflicts', 'Complaint resolution', 'Information availability', 'Percentage of product that hits target market', 'Volume/Weight of product sold', 'Cash received', 'Overhead cost per product sold', 'Economic value add (EVA)', 
                  'Cost of capital (COC)', 'Variation of operating profit', 'Variation of financial expenses', 'Indirect costs', 'Selling price', 'Selling price relative to competitors', 'Product replacement rate', 'Productivity', 'Crop/resource yield', 'Animal condition score', 'Feed units per animal', 'Yeild of forage', 'Crop rotation', 'Storage loss',
                   'Building utilisation', 'Equipment utilisation', 'Labour requirements', 'Strategic segment development', 'Number of new cultivation techniques per year', 'Seedling success rate', 'Production success rate', 'Demand planning accuracy', 'Proportion of products defective', 'Improvement proposals', 'Administration costs', 'Selling price', 
                   'Selling price relative to competitors', 'Customer lost', 'Strategic segment development', 'Number of varieties of products on the market', 'Proportion of products defective', 'Customer offerings', 'Customer care calls', 'Number of community conflicts', 'Complaint resolution', 'Customer satisfaction', 'Number of active customers on database',
                    'Customer loyalty', 'Customer lost', 'Turnover by customer loyalty', 'Customer complaints to total customers', 'Customer satisfaction', 'Customer loyalty', 'Number of required mentor visits', 'Customer complaints to total customers', 'Perceived quality', 'Hygiene and sanitary standards', 'Environmental complaints', 
                    'Customer care calls', 'Complaint resolution', 'Health and safety breaches', 'Customer satisfaction', 'Perceived quality', 'Demand', 'Demand satisfaction rate', 'Delivery time', 'On time delivery', 'Perceived quality', 'Crop/resource yield', 'Hygiene and sanitary standards', 'Demand planning accuracy',
                     'Proportion of products defective', 'Delivery time', 'On time delivery', 'Selling price', 'Selling price relative to competitors', 'Customer offerings', 'Customer care calls', 'Number of community conflicts', 'Complaint resolution', 'Financial returns', 'Cash margin', 'Customer satisfaction', 'Percentage of product that hits target market',
                      'Volume/Weight of product sold', 'Revenue per employee', 'Product replacement rate', 'Productivity', 'Crop/resource yield', 'Feed units per animal', 'Yeild of forage', 'Crop rotation', 'Equipment utilisation', 'Labour requirements', 'Seedling success rate', 'Production success rate', 'Proportion of products defective', 'Resource consumption', 
                      'Customer satisfaction', 'Volume/Weight of product sold', 'Customer loyalty', 'Perceived quality', 'Crop/resource yield', 'Crop rotation', 'Feed units per animal', 'Feed units per animal', 'Yeild of forage', 'Storage loss', 'Building utilisation', 'Financial returns', 'Volume/Weight of product sold', 'Cost drivers', 'Return on investment (ROI)',
                       'Return on Assets (ROA)', 'Cost of capital (COC)', 'Increase turnover', 'Indirect costs', 'Productivity', 'Equipment utilisation', 'Revenue per employee', 'Indirect costs', 'Productivity', 'Crop/resource yield', 'Hygiene and sanitary standards', 'Feed units per animal', 'Yeild of forage', 'Crop rotation', 'Equipment utilisation', 
                       'Labour requirements', 'Number of new cultivation techniques per year', 'Improvement proposals', 'Employee satisfaction', 'Education opportunities', 'Staff development', 'Strategic segment development', 'Number of new selling methods per year', 'Improvement proposals', 'Education opportunities', 'Strategic segment development',
                        'Number of new cultivation techniques per year', 'Number of new selling methods per year', 'Improvement proposals', 'Education opportunities', 'Information availability', 'The length of courses', 'Staff development', 'Resource consumption', 'Environmental complaints', 'Number of community conflicts', 'Complaint resolution',
                         'Information availability', 'Volume/Weight of product sold', 'Economic value add (EVA)', 'Increase turnover', 'Indirect costs', 'Productivity', 'Perceived quality', 'Crop/resource yield', 'Animal condition score', 'Yeild of forage', 'Crop rotation', 'Labour requirements', 'Seedling success rate', 'Production success rate', 
                         'Resource consumption', 'Customer satisfaction', 'Percentage of product that hits target market', 'Volume/Weight of product sold', 'Customer loyalty', 'Product replacement rate', 'Turnover by customer loyalty', 'Total rejected products to total revenue', 'Customer complaints to total customers', 'Perceived quality',
                          'Number of varieties of products on the market', 'Number of new cultivation techniques per year', 'Seedling success rate', 'Demand', 'Demand satisfaction rate', 'Delivery time', 'On time delivery', 'Productivity', 'Crop/resource yield', 'Production success rate', 'Demand planning accuracy', 'Proportion of products defective', 
                          'Customer care calls', 'Complaint resolution', 'Information availability', 'Number of new selling methods per year', 'Sales from new selling methods', 'Information availability', 'Customer loyalty', 'Percentage of new customers', 'Customer lost', 'Turnover by customer loyalty', 'Total rejected products to total revenue', 
                          'Customer complaints to total customers', 'Demand satisfaction rate', 'Environmental complaints', 'Customer offerings', 'Customer care calls', 'Number of community conflicts', 'Complaint resolution', 'Employee satisfaction', 'Number of sales proposals per month', 'Crop/resource yield', 'Yeild of forage', 'Crop rotation', 
                          'Strategic segment development', 'Number of varieties of products on the market', 'Number of new cultivation techniques per year', 'Number of new selling methods per year', 'Sales from new selling methods', 'Information availability', 'The length of courses', 'Employee weekends off', 'Staff development', 'Employee satisfaction',
                           'Education opportunities', 'Information availability', 'Employee weekends off', 'Staff development', 'Information availability', 'Information availability', 'The length of courses', 'Staff development', 'Information availability', 'The length of courses', 'Staff development', 'Health and safety breaches', 'Information availability', 
                           'The length of courses', 'Staff development', 'Health and safety breaches', 'Information availability', 'The length of courses', 'Employee weekends off', 'Staff development', 'Health and safety breaches', 'Financial returns', 'Cash margin', 'Number of active customers on database', 'Volume/Weight of product sold', 'Cash received', 
                           'Income statement', 'Revenue per employee', 'Return on investment (ROI)', 'Invesment portfolio', 'Return on Assets (ROA)', 'Operating profit margin (OPM)', 'Economic value add (EVA)', 'Information availability', 'Employee weekends off', 'Staff development', 'Health and safety breaches')  
 



    }).set_index('objective_measure_relationship_id')
   write(con=con, df=df, table_name='objective_measure')
   log.info("--- Created base objective-measure relationship ---")




def populate_product_store(con=None):

    df = pd.DataFrame({
        "product_id": ("cabbage", "kale", "beans", "spinach", "pumpkin", "strawberry" , "rosemary", "chives") ,
        "product_description": ("plain cabbage", "southern kale", "broad beans","spinach", "pumpkin", "strawberry",
        "rosemary", "chives")




    }).set_index('product_id')
    write(con=con, df=df, table_name='product_store')
    log.info("--- Created base product store table ---")


def populate_resource_type(con=None):

    df = pd.DataFrame(
        { "resource_type_id": ('water','water','finance','plant_nutrients',
                                'animal_feed','animal_feed','seed',
                                'fertilizer','soil','mentor',
                                'finance'),
        "resource_type": ('water_tank', 'borehole','cash' ,'plant_food',
                            'seeds', 'hay_and_grass', 'crop_seeds',
                            'soil_fertilizer', 'top_soil', 'mentor', 'bank_account' ),
        "source": ('municipal', 'borehole','government', 'store',
        'store','store', 'store', 'store','store','government', 'donor' )

        }


    ).set_index('resource_type_id')
    write(con=con, df=df, table_name='resource_type')
    log.info("--- Created base resource type table ---")

def run(con=None):
    try:
        populate_measure_type(con=con)
        populate_perspectives(con=con)
        populate_entity_type(con=con)
        populate_sensor_store(con=con)
        populate_objective_store(con=con)
        populate_measure_store(con=con)
        populate_measure_sensor(con=con)
        populate_objective_measure(con=con)
        populate_product_store(con=con)
        populate_resource_type(con=con)
    except ValueError as e:
        print(e)


if __name__ == '__main__':
    parser = argparse.ArgumentParser('Populating the database')

    args = parser.parse_args()
    db_obj = DbSession(**settings.DATABASE)
    session = db_obj.session
    engine = db_obj.engine
    run(con=engine)



from flask_login import UserMixin
from flask_login import  LoginManager

class UserUtils(UserMixin):
    def __init__(self, id_, name, email, profile_pic):
        self.id = id_
        self.name = name
        self.email = email
        self.profile_pic = profile_pic

    @staticmethod
    def get(user_id, db_session):
        user = db_session.execute(
            "SELECT * FROM public.users WHERE id = '{}'".format(user_id)
        ).fetchone()
        if not user:
            return None

        user = UserUtils(
            id_=user[0], name=user[1], email=user[2], profile_pic=user[3]
        )
        return user

    @staticmethod
    def create(id_, name, email, profile_pic, db_session):
        db_session.execute(
            "INSERT INTO public.users (id, name, email, profile_pic) "
            "VALUES ('{}', '{}', '{}', '{}')".format(id_, name, email, profile_pic)
        )
        db_session.commit()



# coding: utf-8
from sqlalchemy import CHAR, Column, Float, ForeignKey, ForeignKeyConstraint, Index, Integer, String, Time,text
from sqlalchemy.orm import relationship
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.sql.expression import null
from sqlalchemy.sql.sqltypes import TIMESTAMP, Boolean
import flask_login
from flask_login import UserMixin
from db_init_file import db
from sqlalchemy.schema import UniqueConstraint

Base = declarative_base()
metadata = Base.metadata


class EntityType(db.Model):
    __tablename__ = 'entity_type'

    entity_type_id = Column(CHAR(50), primary_key=True)
    entity_type_description = Column(CHAR(100), nullable=False)


class MeasureType(db.Model):
    __tablename__ = 'measure_type'

    measure_type_id = Column(CHAR(50), primary_key=True)
    measure_type_description = Column(CHAR(100), nullable=False)


class Mentor(db.Model):
    __tablename__ = 'mentor'

    mentor_id = Column(CHAR(100), primary_key=True)
    name = Column(CHAR(100), nullable=False)
    surname = Column(CHAR(100), nullable=False)


class Perspective(db.Model):
    __tablename__ = 'perspective'

    perspective_id = Column(CHAR(50), primary_key=True)
    perspective_description = Column(CHAR(100), nullable=False)


class SensorStore(db.Model):
    __tablename__ = 'sensor_store'

    sensor_store_catalog_id = Column(Integer, primary_key=True, nullable=False)
    sensor_store_id = Column(CHAR(100), primary_key=True, nullable=False)
    sensor_description = Column(CHAR(100), nullable=False)
    sensor_active_indicator = Column(CHAR(50), nullable=False)


class Farm(db.Model):
    __tablename__ = 'farm'

    farm_id = Column(CHAR(100), primary_key=True)
    farm_type = Column(CHAR(100), nullable=False)
    longitude = Column(CHAR(100), nullable=False)
    lattitude = Column(CHAR(100), nullable=False)
    size = Column(Integer, nullable=False)
    entity_type_id = Column(ForeignKey('entity_type.entity_type_id'), nullable=False, index=True)

    entity_type = relationship('EntityType')


class MeasureStore(db.Model):
    __tablename__ = 'measure_store'

    measure_store_catalog_id = Column(Integer, primary_key=True, nullable=False)
    measure_store_id = Column(CHAR(100), primary_key=True, nullable=False)
    measure_description = Column(CHAR(100), nullable=False)
    measure_type_id = Column(ForeignKey('measure_type.measure_type_id'), nullable=False, index=True)

    measure_type = relationship('MeasureType')


class ObjectiveStore(db.Model):
    __tablename__ = 'objective_store'

    objective_id = Column(CHAR(100), primary_key=True)
    objective_description = Column(CHAR(100), nullable=False)
    perspective_id = Column(ForeignKey('perspective.perspective_id'), nullable=False, index=True)

    perspective = relationship('Perspective')


class Customer(db.Model):
    __tablename__ = 'customer'

    customer_id = Column(CHAR(100), primary_key=True)
    farm_id = Column(ForeignKey('farm.farm_id'), nullable=False, index=True)
    customer_name = Column(CHAR(100), nullable=False)

    farm = relationship('Farm')

class CustomerTransaction(db.Model):
    __tablename__ = 'customer_transactions'

    transaction_id = Column(Integer, primary_key=True)
    product_id = Column(ForeignKey('product_store.product_id'), nullable=False, index=True)
    customer_id = Column(ForeignKey('customer.customer_id'), nullable=False, index=True)
    farm_id = Column(ForeignKey('farm.farm_id'), nullable=False, index=True)

    customer = relationship('Customer')
    farm = relationship('Farm')
    product = relationship('ProductStore')

class ProductStore(db.Model):
    __tablename__ = 'product_store'
    product_id = Column(CHAR(100), primary_key=True)
    product_description = Column(CHAR(100), nullable=False)


class FarmProduct(db.Model):
    __tablename__ = 'farm_product'

    farm_product_catalog_id = Column(Integer, primary_key=True)
    product_id = Column(ForeignKey('product_store.product_id'), nullable=False, index=True)
    farm_id = Column(ForeignKey('farm.farm_id'), nullable=False, index=True)
    create_time = Column(Time(True), server_default=text("now()"), nullable=False)

    farm = relationship('Farm')
    product = relationship('ProductStore')
    

    
class Employee(db.Model):
    __tablename__ = 'employee'

    employee_id = Column(CHAR(100), primary_key=True)
    name = Column(CHAR(100), nullable=False)
    surname = Column(CHAR(100), nullable=False)
    farm_id = Column(ForeignKey('farm.farm_id'), nullable=False, index=True)

    farm = relationship('Farm')


class FarmMeasure(db.Model):
    __tablename__ = 'farm_measure'
    __table_args__ = (
        ForeignKeyConstraint(['measure_store_catalog_id', 'measure_store_id'], ['measure_store.measure_store_catalog_id', 'measure_store.measure_store_id']),
        Index('fkidx_89', 'measure_store_catalog_id', 'measure_store_id')
    )

    farm_measure_catalog_id = Column(Integer, primary_key=True)
    measure_store_catalog_id = Column(Integer, nullable=False)
    measure_store_id = Column(CHAR(100), nullable=False)
    farm_id = Column(ForeignKey('farm.farm_id'), nullable=False, index=True)

    farm = relationship('Farm')
    measure_store_catalog = relationship('MeasureStore')


class FarmObjective(db.Model):
    __tablename__ = 'farm_objective'

    farm_objective_catalog_id = Column(Integer, primary_key=True)
    objective_id = Column(ForeignKey('objective_store.objective_id'), nullable=False, index=True)
    farm_id = Column(ForeignKey('farm.farm_id'), nullable=False, index=True)

    farm = relationship('Farm')
    objective = relationship('ObjectiveStore')


class FarmSensor(db.Model):
    __tablename__ = 'farm_sensor'
    __table_args__ = (
        ForeignKeyConstraint(['sensor_store_catalog_id', 'sensor_store_id'], ['sensor_store.sensor_store_catalog_id', 'sensor_store.sensor_store_id']),
        Index('fkidx_54', 'sensor_store_catalog_id', 'sensor_store_id')
    )

    sensor_store_id = Column(CHAR(100), primary_key=True, nullable=False)
    create_time = Column(Time(True), server_default=text("now()"), nullable=False)
    farm_id = Column(ForeignKey('farm.farm_id'), primary_key=True, nullable=False, index=True)
    sensor_store_catalog_id = Column(Integer, primary_key=True, nullable=False)
    sensor_value_lower_bound = Column(Float,nullable=False )
    sensor_value_upper_bound = Column(Float,nullable=True )
    sensor_value_ideal = Column(Float,nullable=True)
    farm = relationship('Farm')
    sensor_store_catalog = relationship('SensorStore')


class SensorForecast(FarmSensor):
    __tablename__ = 'sensor_forecast'
    __table_args__ = (
        ForeignKeyConstraint(['sensor_store_id', 'farm_id', 'sensor_store_catalog_id'], ['farm_sensor.sensor_store_id', 'farm_sensor.farm_id', 'farm_sensor.sensor_store_catalog_id']),
        Index('fkidx_70', 'sensor_store_id', 'farm_id', 'sensor_store_catalog_id')
    )

    sensor_store_catalog_id = Column(Integer, primary_key=True, nullable=False)
    farm_id = Column(CHAR(100), primary_key=True, nullable=False)
    sensor_store_id = Column(CHAR(100), primary_key=True, nullable=False)
    forecast_value = Column(String(50), nullable=False)
    forecast_value_upper = Column(Float, nullable=False)
    forecast_value_lower = Column(Float, nullable=False)
    forecast_date = Column(Float, nullable=False)


class Finance(db.Model):
    __tablename__ = 'finance'

    tran_id = Column(Integer, primary_key=True)
    farm_id = Column(ForeignKey('farm.farm_id'), nullable=False, index=True)
    income = Column(Float, nullable=True)
    expense = Column(Float, nullable=True)
    returns = Column(Float, nullable=True)
    expected_revenue = Column(Float, nullable=False)
    date = Column(Time(True), server_default=text("now()"), nullable=False)

    farm = relationship('Farm')


class MeasureSensor(db.Model):
    __tablename__ = 'measure_sensor'
    __table_args__ = (
        ForeignKeyConstraint(['measure_store_catalog_id', 'measure_store_id'], ['measure_store.measure_store_catalog_id', 'measure_store.measure_store_id']),
        ForeignKeyConstraint(['sensor_store_catalog_id', 'sensor_store_id'], ['sensor_store.sensor_store_catalog_id', 'sensor_store.sensor_store_id']),
        Index('fkidx_78', 'measure_store_catalog_id', 'measure_store_id'),
        Index('fkidx_82', 'sensor_store_catalog_id', 'sensor_store_id')
    )

    measure_sensor_relationship_id = Column(Integer, primary_key=True)
    measure_store_catalog_id = Column(Integer, nullable=False)
    measure_store_id = Column(CHAR(100), nullable=False)
    sensor_store_catalog_id = Column(Integer, nullable=False)
    sensor_store_id = Column(CHAR(100), nullable=False)

    measure_store_catalog = relationship('MeasureStore')
    sensor_store_catalog = relationship('SensorStore')


class MentorComment(db.Model):
    __tablename__ = 'mentor_comment'

    comment_id = Column(Integer, primary_key=True)
    comment = Column(CHAR(100), nullable=False)
    farm_id = Column(ForeignKey('farm.farm_id'), nullable=False, index=True)
    mentor_id = Column(ForeignKey('mentor.mentor_id'), nullable=False, index=True)

    farm = relationship('Farm')
    mentor = relationship('Mentor')


class ObjectiveMeasure(db.Model):
    __tablename__ = 'objective_measure'
    __table_args__ = (
        ForeignKeyConstraint(['measure_store_catalog_id', 'measure_store_id'], ['measure_store.measure_store_catalog_id', 'measure_store.measure_store_id']),
        Index('fkidx_128', 'measure_store_catalog_id', 'measure_store_id')
    )

    objective_measure_relationship_id = Column(Integer, primary_key=True, nullable=False)
    objective_id = Column(ForeignKey('objective_store.objective_id'), nullable=False, index=True)
    measure_store_catalog_id = Column(Integer, nullable=False)
    measure_store_id = Column(CHAR(100), nullable=False)

    measure_store_catalog = relationship('MeasureStore')
    objective = relationship('ObjectiveStore')


class ObjectiveSensor(db.Model):
    __tablename__ = 'objective_sensor'
    __table_args__ = (
        ForeignKeyConstraint(['sensor_store_catalog_id', 'sensor_store_id'], ['sensor_store.sensor_store_catalog_id', 'sensor_store.sensor_store_id']),
        Index('fkidx_115', 'sensor_store_catalog_id', 'sensor_store_id')
    )

    objective_sensor_catalog_id = Column(Integer, primary_key=True)
    sensor_store_catalog_id = Column(Integer, nullable=False)
    sensor_store_id = Column(CHAR(100), nullable=False)
    objective_id = Column(ForeignKey('objective_store.objective_id'), nullable=False, index=True)

    objective = relationship('ObjectiveStore')
    sensor_store_catalog = relationship('SensorStore')


class EmployeeStatus(db.Model):
    __tablename__ = 'employee_status'

    employee_id = Column(ForeignKey('employee.employee_id'), primary_key=True, nullable=False, index=True)
    farm_id = Column(ForeignKey('farm.farm_id'), primary_key=True, nullable=False, index=True)
    date = Column(Time(True), primary_key=True, server_default=text("now()") ,nullable=False)
    employee_satisfaction = Column('employee satisfaction', Integer, nullable=False)
    complaint = Column(CHAR(180))
    comment = Column(CHAR(180), nullable=False)

    employee = relationship('Employee')
    farm = relationship('Farm')

class SensorDatum(db.Model):
    __tablename__ = 'sensor_data'
    __table_args__ = (
        ForeignKeyConstraint(['sensor_store_id', 'farm_id', 'sensor_store_catalog_id'], ['farm_sensor.sensor_store_id', 'farm_sensor.farm_id', 'farm_sensor.sensor_store_catalog_id']),
        Index('fkidx_202', 'sensor_store_id', 'farm_id', 'sensor_store_catalog_id')
    )

    sensor_store_id = Column(CHAR(100), primary_key=True, nullable=False)
    farm_id = Column(CHAR(100), primary_key=True, nullable=True)
    sensor_store_catalog_id = Column(Integer, primary_key=True, nullable=False)
    row_id = Column(Float, primary_key=True, nullable=False)
    sensor_value = Column(Float, nullable=False)
    create_time = Column(TIMESTAMP(True), nullable=False)

    sensor_store = relationship('FarmSensor')


class User(db.Model, UserMixin):
    __tablename__ = 'users'

    id = db.Column(db.String(160), primary_key=True, nullable=False)
    name = db.Column(db.String(160), nullable=False)
    email = db.Column(db.String(240), nullable=False)
    profile_pic = db.Column(db.String(160), nullable=False)


class ResourceType(db.Model):
    __tablename__ = 'resource_type'

    resource_type_id = Column(CHAR(100), primary_key=True, nullable=False)
    resource_type = Column(CHAR(100), primary_key=True, nullable=False)
    source = Column(CHAR(100), primary_key=True, nullable=False)

class ResourceStore(db.Model):
    __tablename__ = 'resource_store'
    __table_args__ = (
        ForeignKeyConstraint(['resource_type_id', 'resource_type', 'source'], ['resource_type.resource_type_id', 'resource_type.resource_type', 'resource_type.source']),
        Index('fkidx_268', 'resource_type_id', 'resource_type', 'source')
    )

    resource_store_catalog_id = Column(Integer, primary_key=True, nullable=False)
    resource_store_id = Column(CHAR(100), primary_key=True, nullable=False)
    resource_description = Column(CHAR(100), nullable=False)
    farm_id = Column(ForeignKey('farm.farm_id'), index=True, nullable=True)
    resource_type_id = Column(CHAR(100), nullable=False)
    resource_type = Column(CHAR(100), nullable=False)
    source = Column(CHAR(100), nullable=False)
    capacity = Column(Float, nullable=False)
    can_split = Column(Boolean, nullable=False)

    farm = relationship('Farm')
    resource_type1 = relationship('ResourceType')

class FarmResource(db.Model):
    __tablename__ = 'farm_resource'
    __table_args__ = (
        ForeignKeyConstraint(['resource_store_catalog_id', 'resource_store_id'], ['resource_store.resource_store_catalog_id', 'resource_store.resource_store_id']),
        Index('fkidx_289', 'resource_store_catalog_id', 'resource_store_id')
    )

    farm_resource_relationship_id = Column(Integer, primary_key=True)
    resource_store_catalog_id = Column(Integer, nullable=False)
    resource_store_id = Column(CHAR(100), nullable=False)
    farm_id = Column(ForeignKey('farm.farm_id'), nullable=False, index=True)

    farm = relationship('Farm')
    resource_store_catalog = relationship('ResourceStore')

class SensorResource(db.Model):
    __tablename__ = 'sensor_resource'
    __table_args__ = (
        ForeignKeyConstraint(['resource_store_catalog_id', 'resource_store_id'], ['resource_store.resource_store_catalog_id', 'resource_store.resource_store_id']),
        ForeignKeyConstraint(['sensor_store_catalog_id', 'sensor_store_id'], ['sensor_store.sensor_store_catalog_id', 'sensor_store.sensor_store_id']),
        Index('fkidx_280', 'resource_store_catalog_id', 'resource_store_id'),
        Index('fkidx_299', 'sensor_store_catalog_id', 'sensor_store_id')
    )

    resource_store_catalog_id = Column(Integer, nullable=False)
    resource_store_id = Column(CHAR(100), nullable=False)
    sensor_store_catalog_id = Column(Integer, nullable=False)
    sensor_store_id = Column(CHAR(100), nullable=False)
    sensor_resource_relationship_id = Column(Integer, primary_key=True)

    resource_store_catalog = relationship('ResourceStore')
    sensor_store_catalog = relationship('SensorStore')

class ResourceDatum(db.Model):
    __tablename__ = 'resource_data'
    __table_args__ = (
        ForeignKeyConstraint(['sensor_store_id', 'farm_id', 'sensor_store_catalog_id', 'row_id_1'], ['sensor_data.sensor_store_id', 'sensor_data.farm_id', 'sensor_data.sensor_store_catalog_id', 'sensor_data.row_id']),
        Index('fkidx_308', 'sensor_store_id', 'farm_id', 'sensor_store_catalog_id', 'row_id_1')
    )

    sensor_store_id = Column(CHAR(100), nullable=False)
    farm_id = Column(CHAR(100), nullable=True)
    sensor_store_catalog_id = Column(Integer, nullable=False)
    row_id_1 = Column(Float, nullable=False)
    farm_resource_relationship_id = Column(ForeignKey('farm_resource.farm_resource_relationship_id'), nullable=False, index=True)
    row_id = Column(Integer, primary_key=True)

    farm_resource_relationship = relationship('FarmResource')
    sensor_store = relationship('SensorDatum')    


class State(db.Model):
    __tablename__ = 'state'

    ID = Column(CHAR(20), nullable=False, primary_key=True)
    Name = Column(CHAR(20), nullable=False)


class Stakeholder(db.Model):
    __tablename__ = 'stakeholder'
    __table_args__ = (
        ForeignKeyConstraint(['resource_store_catalog_id', 'resource_store_id'], ['resource_store.resource_store_catalog_id', 'resource_store.resource_store_id']),
        Index('fk_324', 'resource_store_catalog_id', 'resource_store_id')
    )

    stakeholder_id = Column(Integer, primary_key=True)
    stakeholder_name = Column(CHAR(100), nullable=False)
    resource_store_catalog_id = Column(Integer)
    farm_id = Column(ForeignKey('farm.farm_id'), index=True)
    resource_store_id = Column(CHAR(100))
    stakeholder_type = Column(CHAR(100), nullable=False)

    farm = relationship('Farm')
    resource_store_catalog = relationship('ResourceStore')


from setuptools import setup
from pathlib import Path
import bsc.__init__

with open('requirements.txt') as req:
    requirements = list(filter(None, req.read().split('\n')))

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

version = bsc.__init__.__version__
setup(
    name='python-bsc',
    packages=['bsc'],
    version='0.0.3alpha',
    license='MIT',
    description='A python based balanced scorecard implementation for managing objectives, tracking KPIs and improving organisational performance',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Carl du Plessis',
    author_email='',
    url='https://github.com/PandaCarl/python-bsc',  #
    download_url='https://github.com/PandaCarl/python-bsc/archive/refs/tags/0.0.3alpha.tar.gz',
    keywords=['Balanced Scorecard', "KPI's", 'Performance management'],
    install_requires=requirements,
    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Developers',  # Define that your audience are developers
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',  # Again, pick a license
        'Programming Language :: Python :: 3'
    ],
)



from datetime import datetime, timezone
from flask_sqlalchemy import SQLAlchemy
from flask import Flask,flash,jsonify, render_template, redirect, url_for, request
from flask_bootstrap import Bootstrap
from flask_wtf import FlaskForm, Form, form
from sqlalchemy.engine.base import Transaction
from sqlalchemy.sql.expression import label
from wtforms import HiddenField, StringField,FloatField ,SubmitField, DateField, IntegerField, DateTimeField, SelectField, RadioField
from wtforms.fields.core import BooleanField
from wtforms.validators import DataRequired, Optional
import requests
from db_init_file import db
from db_classes import FarmProduct, Mentor, Perspective,Customer,Employee, FarmObjective, FarmSensor, SensorForecast, Finance, \
    MentorComment,EmployeeStatus ,  EntityType, Farm, MeasureType, ObjectiveSensor, MeasureSensor, FarmMeasure, Stakeholder, User, MeasureStore, ObjectiveStore, SensorStore, \
    ObjectiveMeasure, Finance, ProductStore, ResourceType, ResourceStore, ResourceDatum, FarmResource, SensorResource, State, County, Stakeholder

import os
import flask_login
from flask_login import LoginManager, current_user, login_required, login_user, logout_user
from oauthlib.oauth2 import WebApplicationClient
import json
from user import UserUtils
import pandas as pd
import json
import plotly
import plotly.express as px

import jsonify

import logging
log = logging.getLogger()


# Configuration
GOOGLE_CLIENT_ID = os.environ.get("GOOGLE_CLIENT_ID", None)
GOOGLE_CLIENT_SECRET = os.environ.get("GOOGLE_CLIENT_SECRET", None)
GOOGLE_DISCOVERY_URL = (
    "https://accounts.google.com/.well-known/openid-configuration"
)

# Flask app setup
app = Flask(__name__)
app.secret_key = os.environ.get("SECRET_KEY") or os.urandom(24)

# OAuth 2 client setup
client = WebApplicationClient(GOOGLE_CLIENT_ID)


app = Flask(__name__)
db_password=os.environ["DB_PASS"]
app.config['SQLALCHEMY_DATABASE_URI'] = f"postgresql://postgres:{db_password}@localhost:<port>/<db_name>"
app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
app.config['SECRET_KEY'] = ''
db.app = app
db.init_app(app)
Bootstrap(app)
db.create_all()


# User session management setup
# https://flask-login.readthedocs.io/en/latest
login_manager = LoginManager()
login_manager.init_app(app)

def get_google_provider_cfg():
    return requests.get(GOOGLE_DISCOVERY_URL).json()


# Flask-Login helper to retrieve a user from our db
@login_manager.user_loader
def load_user(user_id):
    return UserUtils.get(user_id, db.session)

class AddStakeholder(FlaskForm):
    listed_farm = db.session.query(Farm.farm_id).all()
    listed_resources = db.session.query(ResourceStore.resource_store_id).all()
    stakeholder_type = SelectField(label='Stakeholder type', choices=['Governing body', 'Donor', 'Investor', 'Partner company'], validators=[DataRequired()])
    stakeholder_name = StringField(label="Stakeholder name", validators=[DataRequired()])
    stakeholder_farm = SelectField(label="Farm", choices= [value for value, in listed_farm], validators=[DataRequired()])
    stakeholder_resources = SelectField(label='Resources', choices=[value for value, in listed_resources],validators=[DataRequired()])
    submit = SubmitField("Register a stakeholder")

class AddFarmer(FlaskForm):
    listed_farm = db.session.query(Farm.farm_id).all()
    farmer_name = StringField(label="Farmer name", validators=[DataRequired()])
    farmer_surname = StringField(label="Farmer surname", validators=[DataRequired()])  
    farm = SelectField(label='Select farm', choices= [value for value, in listed_farm], validators=[DataRequired()])
    submit = SubmitField('Register farmer')


class AddFarm(FlaskForm):
    entity_choices = db.session.query(EntityType.entity_type_id).all()
    entity_choices = [value for value, in entity_choices]
    city = StringField(label='City', validators=[DataRequired()])
    farm_type = SelectField(label="Type", choices=entity_choices, validators=[DataRequired()])
    longitude = StringField(label="longitude", validators=[DataRequired()])
    lattitude = StringField(label='lattitude', validators=[DataRequired()])
    size = IntegerField(label="Size in square meters", validators=[DataRequired()])
    submit = SubmitField(label='Add farm')


class AddSensor(FlaskForm):
    sensor_list = db.session.query(SensorStore.sensor_store_id).all()
    sensor_list = [value for value, in sensor_list]
    sensor = SelectField(label="Sensor", choices=sensor_list, validators=[DataRequired()])
    sensor_lower_bound = FloatField(label='Sensor lower bound value', validators=[DataRequired()])
    sensor_upper_bound = FloatField(label='Sensor upper bound value',validators=[Optional()])
    sensor_ideal_value = FloatField(label='Sensor ideal value', validators=[Optional()])
    submit = SubmitField('Submit')


class AddFarmMeasure(FlaskForm):

    measures = [value for value, in db.session.query(MeasureStore.measure_store_id).all()]
    measure = SelectField(label="Measure", choices=measures, validate_choice=[DataRequired()])
    submit = SubmitField('Submit')

class AddFarmObjective(FlaskForm):
    objective_types = [value for value, in db.session.query(Perspective.perspective_id).all()]
    objectives = [value for value, in db.session.query(ObjectiveStore.objective_id).all()]
    perspective = SelectField(label="Perspective/Objective type", choices=objective_types, validate_choice=[DataRequired()])
    objective = SelectField(label="Objective", choices=objectives, validate_choice=[DataRequired()])
    submit = SubmitField('Submit')



class AddResource(FlaskForm):
    ## Adds resource items to the resource store
    resource_types = [value for value, in db.session.query(ResourceType.resource_type).all()]
    farms = [value for value, in db.session.query(Farm.farm_id).all()]
    sources = [value for value, in db.session.query(ResourceType.source).all()]
    # sources = list(['Government', 'Donor', 'Natural', 'Self generated'])
    resource_type = SelectField(label='Resource type', choices=resource_types, validate_choice=[DataRequired()])
    resource_description = StringField(label='Resource description')
    farm = SelectField(label='Farm', choices=farms, default=None)
    source = SelectField(label='Source', choices=sources, validate_choice=[DataRequired()])
    capacity = FloatField(label='Source capacity (Rand value, kg, mm etc.)')
    can_split = BooleanField(label='Can this source be split?' )

    submit = SubmitField('Submit')


class AddFarmProduct(FlaskForm):

    products = [value for value, in db.session.query(ProductStore.product_id).all()]
    product = SelectField(label='Product', choices=products, validate_choice=[DataRequired()])
    submit = SubmitField("Submit")


class AddCustomer(FlaskForm):
    customer_name = StringField(label="Customer Name")

    submit = SubmitField('Submit')

class Transaction(FlaskForm):
    products = [value for value, in db.session.query(ProductStore.product_id).all()]
    products_bought = SelectField(label='Product', choices=products, validate_choice=[DataRequired()]) 


class AddFinance(FlaskForm):
    income = FloatField(label='Income', validators=[DataRequired()] )
    expenses = FloatField(label='Expenses', validators=[DataRequired()] )
    returns = FloatField(label='Returns', validators=[DataRequired()] )
    submit = SubmitField('Add Finances')

class LinkResource(FlaskForm):
    resources = [value for value, in db.session.query(ResourceStore.resource_store_id).all()]
    resource = SelectField(label='Resource', choices=resources, validate_choice=[DataRequired()])
    submit = SubmitField('Link Resource')

@app.route("/home", methods=['GET', 'POST'])
# @login_required
def home():
    all_farms = db.session.query(Farm).all()
    return render_template("home.html", farms=all_farms)



@app.route("/add_farmer", methods=['GET', 'POST'])
def add_farmer():
    new_farmer = AddFarmer()
    if request.method == "GET":
        farm_id = request.args['id']
        farm_employees = db.session.query(Employee).filter(Employee.farm_id==farm_id).all()

    if new_farmer.validate_on_submit():
        try:
            employee_id = int(max([value for value, in db.session.query(Employee.employee_id).all()])) + 1
        except:
            employee_id = 1
        farmer = Employee(employee_id=employee_id,
                        name=new_farmer.farmer_name.data,
                        surname=new_farmer.farmer_surname.data,
                        farm_id=new_farmer.farm.data)
        db.session.add(farmer)
        db.session.commit()
        
        return redirect(url_for('home'))
    return render_template("add_farmer.html", form=new_farmer, farmers=farm_employees, farm=farm_id)


@app.route("/add_stakeholder", methods=["GET","POST"])
def add_stakeholder():
    new_stakeholder = AddStakeholder()
    if request.method == "GET":
        stakeholders = db.session.query(Stakeholder.stakeholder_name, Stakeholder.stakeholder_type, Stakeholder.resource_store_id, Stakeholder.farm_id).all()
    
    if new_stakeholder.validate_on_submit():
        try:
            stakeholder_id = int(max([value for value, in db.session.query(Stakeholder.stakeholder_id).all()])) + 1
        except:
            stakeholder_id = 1
        
        stakeholder_resource_store_catalog_id = db.session.query(ResourceStore.resource_store_catalog_id).filter(ResourceStore.resource_store_id==new_stakeholder.stakeholder_resources.data)
        stakeholder = Stakeholder(stakeholder_id=stakeholder_id,
                                stakeholder_name=new_stakeholder.stakeholder_name.data,
                                resource_store_catalog_id=stakeholder_resource_store_catalog_id,
                                farm_id=new_stakeholder.stakeholder_farm.data,
                                resource_store_id = new_stakeholder.stakeholder_resources.data,
                                stakeholder_type=new_stakeholder.stakeholder_type.data)

        db.session.add(stakeholder)
        db.session.commit()
    
        return redirect(url_for('home'))
    # stakeholders = db.session.query(Stakeholder).all()
    
    return render_template("add_stakeholder.html", form=new_stakeholder, stakeholders=stakeholders)

@app.route("/add_resource", methods=["GET", "POST"])
#@login_required
def add_resource():
    new_resource = AddResource()
    resource_store_data = db.session.query(ResourceStore).all()
    try:
        resource_catalog_id = max([value for value, in db.session.query(ResourceStore.resource_store_catalog_id).all()]) + 1
    except:
        resource_catalog_id = 1
    
    if request.method == 'GET':
        resource_store_data = db.session.query(ResourceStore).all()

#          <th> {% for farm in farms %} <li> {{farm.farm_id}} </li> {% endfor %} </th>
    if new_resource.validate_on_submit():
        resource_store_data = db.session.query(ResourceStore).all()
        resource_catalog_id = resource_catalog_id
        resource_type = new_resource.resource_type.data
        resource_type_id = db.session.query(ResourceType.resource_type_id).filter(ResourceType.resource_type == resource_type)
        resource_description = new_resource.resource_description.data
        farm_id = new_resource.farm.data
        source = new_resource.source.data
        capacity = new_resource.capacity.data
        can_split = new_resource.can_split.data

        additional_resource = ResourceStore(
            resource_store_catalog_id = resource_catalog_id,
            resource_store_id = f"{resource_type.strip()}_{capacity}",
            resource_description = resource_description,
            farm_id = farm_id,
            resource_type_id = resource_type_id,
            resource_type = resource_type,
            source = source,
            capacity = capacity,
            can_split = can_split
        )
        db.session.add(additional_resource)
        db.session.commit()
        return redirect(url_for('home'))

    return render_template("add_resource.html", form=new_resource, resource_store_data=resource_store_data)


@app.route("/link_resource", methods=["GET", "POST"])
def link_resource():
    link_resource = LinkResource()

    all_resources = db.session.query(ResourceStore)
    if request.method == "GET":
        farm_id = request.args['id']
        
        resource_list_for_farm = db.session.query(FarmResource.resource_store_id, 
        ResourceStore.resource_description).filter_by(farm_id=farm_id).join(ResourceStore,
        ResourceStore.resource_store_catalog_id == FarmResource.resource_store_catalog_id).all()


    if link_resource.validate_on_submit():
        farm_id = request.args['id']
        try:
            farm_resource_relationship_id = max([value for value, in db.session.query(FarmResource.farm_resource_relationship_id).all()]) + 1
        except:
            farm_resource_relationship_id = 1
        
        resource_store_id = link_resource.resource.data
        resource_store_catalog_id = db.session.query(ResourceStore.resource_store_catalog_id).filter(ResourceStore.resource_store_id == resource_store_id)

        linked_resource = FarmResource(
            farm_resource_relationship_id = farm_resource_relationship_id,
            resource_store_catalog_id = resource_store_catalog_id,
            resource_store_id=resource_store_id,
            farm_id=farm_id
        )

        db.session.add(linked_resource)
        db.session.commit()
        return redirect(url_for('home'))
    return render_template("link_resource.html", all_resources=all_resources ,resources=resource_list_for_farm, form=link_resource, farm=farm_id)


@app.route("/add_sensor", methods=["GET", "POST"])
#@login_required
def add_sensor():
    new_sensor = AddSensor()
    try:
        sensor_catalog_id = max([value for value, in db.session.query(SensorStore.sensor_store_catalog_id).all()]) + 1
    except:
        sensor_catalog_id = 1

    if request.method == "GET" or 'POST':
        farm_id = request.args['id']
        sensor_list_for_farm = db.session.query(FarmSensor.sensor_store_id, 
        SensorStore.sensor_description).filter_by(farm_id=farm_id).join(SensorStore,
        SensorStore.sensor_store_catalog_id==FarmSensor.sensor_store_catalog_id).all()

    if new_sensor.validate_on_submit():
        # sensor_id = "{}_{}_{}".format(new_sensor.farm_id.data, new_sensor.sensor_type.data, sensor_catalog_id)
        farm_id = request.args['id']
        sensor_store_id = new_sensor.sensor.data
        sensor_store_catalog_id = db.session.query(SensorStore.sensor_store_catalog_id).filter_by(sensor_store_id=new_sensor.sensor.data).first()[0]
        sensor_lower_bound = new_sensor.sensor_lower_bound.data
        sensor_upper_bound = new_sensor.sensor_upper_bound.data
        sensor_ideal_value = new_sensor.sensor_ideal_value.data
        new_sensor = FarmSensor(sensor_store_catalog_id=sensor_store_catalog_id,
                            sensor_store_id=sensor_store_id,
                            farm_id=farm_id,
                            sensor_value_lower_bound=sensor_lower_bound,
                            sensor_value_upper_bound=sensor_upper_bound,
                            sensor_value_ideal=sensor_ideal_value,
                            create_time=datetime.now(timezone.utc)
                           )

        db.session.add(new_sensor)
        db.session.commit()
        # NOTE: You can use the redirect method from flask to redirect to another route
        # e.g. in this case to the home page after the form has been submitted.

        return redirect(url_for('home'))
    return render_template("add_sensor.html", sensors=sensor_list_for_farm, form=new_sensor, farm=farm_id)


@app.route("/add_farm", methods=["GET", "POST"])
#@login_required
def add_farm():
    new_farm = AddFarm()

    if new_farm.validate_on_submit():
        farm_id = "{}_{}_{}".format(new_farm.city.data, new_farm.longitude.data, new_farm.lattitude.data)
        farm_type = new_farm.farm_type.data
        longitude = new_farm.longitude.data
        lattitude = new_farm.lattitude.data
        size = new_farm.size.data
        entity_type = new_farm.farm_type.data
        new_farm = Farm(farm_id=farm_id,
                        farm_type=farm_type,
                        longitude=longitude,
                        lattitude=lattitude,
                        size=size,
                        entity_type_id=entity_type)

        db.session.add(new_farm)
        db.session.commit()
        # NOTE: You can use the redirect method from flask to redirect to another route
        # e.g. in this case to the home page after the form has been submitted.

        return redirect(url_for('home'))
    return render_template("add.html", form=new_farm)


@app.route("/low_info")
#@login_required
def low_info():
    return render_template("low_info.html")

@app.route("/high_info")
#@login_required
def high_info():
    return render_template("high_info.html")


@app.route("/add_measure", methods=["GET", "POST"])
#@login_required
def add_measure():

    new_measure = AddFarmMeasure()

    if request.method == "GET":
        farm_id = request.args['id']  
        farm_measures = db.session.query(FarmMeasure.measure_store_id,
        MeasureStore.measure_description).filter_by(farm_id=farm_id).join(MeasureStore,
     FarmMeasure.measure_store_catalog_id==MeasureStore.measure_store_catalog_id).all()

    if new_measure.validate_on_submit():
        try:
            farm_measure_catalog_id = max([value for value, in db.session.query(FarmMeasure.farm_measure_catalog_id).all()]) + 1
        except:
            farm_measure_catalog_id = 1
        measure_store_catalog_id = db.session.query(MeasureStore.measure_store_catalog_id).filter_by(measure_store_id=new_measure.measure.data).all()[0][0]
        farm_id = request.args['id']
        farm_measures = db.session.query(FarmMeasure).filter_by(farm_id=farm_id).join(MeasureStore, FarmMeasure.measure_store_catalog_id==MeasureStore.measure_store_catalog_id).all()

        measure_store_id = new_measure.measure.data 
        new_measure = FarmMeasure(farm_measure_catalog_id=farm_measure_catalog_id,
                                    measure_store_catalog_id=measure_store_catalog_id,
                                    farm_id=farm_id,
                                    measure_store_id=measure_store_id
                                    )
        db.session.add(new_measure)
        db.session.commit()
        return redirect(url_for('home'))

    return render_template('add_measure.html', farm=farm_id, measures=farm_measures, form=new_measure)
#
#
@app.route("/add_objective", methods=['GET', 'POST'])
#@login_required
def add_objective():

    new_objective = AddFarmObjective()

    if request.method == "GET":
        farm_id = request.args['id']
        objectives = db.session.query(FarmObjective.objective_id, ObjectiveStore.objective_description, ObjectiveStore.perspective_id).join(ObjectiveStore, ObjectiveStore.objective_id==FarmObjective.objective_id).filter(FarmObjective.farm_id==farm_id).all()
        
    if new_objective.validate_on_submit():

        try:
            farm_objective_catalog_id = max([value for value, in db.session.query(FarmObjective.farm_objective_catalog_id).all()]) + 1
        except Exception as e:
            farm_objective_catalog_id = 1
        
        new_objective.objective.choices = [value for value, in db.session.query(ObjectiveStore.objective_id).filter(ObjectiveStore.perspective_id==new_objective.perspective.data)]
        farm_id = request.args['id']
        # farm_id = db.session.query(Farm.farm_id).filter_by(farm_id=farm_id).all()[0][0]
        objectives = db.session.query(FarmObjective.objective_id, ObjectiveStore.objective_description, ObjectiveStore.perspective_id).join(ObjectiveStore, ObjectiveStore.objective_id==FarmObjective.objective_id).filter(FarmObjective==farm_id).all()
        new_objective = FarmObjective(farm_objective_catalog_id=farm_objective_catalog_id,
                                    objective_id=new_objective.objective.data,
                                    farm_id=farm_id)
        db.session.add(new_objective)
        db.session.commit()
        return redirect(url_for('home'))

    return render_template('add_objective.html', farm=farm_id, objectives=objectives, form=new_objective)


@app.route("/farm_overview",methods=['GET', 'POST'])
def farm_overview():
    if request.method == "GET":
        farm_id = request.args['id']
        farm_measures = db.session.query(FarmMeasure).filter(FarmMeasure.farm_id==farm_id).all()
        farm_sensors = db.session.query(FarmSensor).filter(FarmSensor.farm_id==farm_id).all()
        farm_objectives = db.session.query(FarmObjective).filter(FarmObjective.farm_id==farm_id).all()
        farm_products = db.session.query(FarmProduct).filter(FarmProduct.farm_id==farm_id).all()
        farm_resources = db.session.query(FarmResource).filter(FarmResource.farm_id==farm_id).all()
        farmers = db.session.query(Employee).filter(Employee.farm_id==farm_id).all()
    return render_template('overview.html',
                                            farm_id=farm_id,
                                            farm_measures=farm_measures,
                                            farm_sensors=farm_sensors,
                                            farm_objectives=farm_objectives,
                                            farm_products=farm_products,
                                            farm_resources=farm_resources,
                                            farmers=farmers)
    # return redirect(url_for("home"))



@app.route("/add_products", methods=['GET', 'POST'])
def add_products():
    new_product = AddFarmProduct()

    if request.method == "GET":
        farm_id = request.args['id']
        products = db.session.query(FarmProduct.product_id, ProductStore.product_description).join(ProductStore, ProductStore.product_id == FarmProduct.product_id).filter(FarmProduct.farm_id==farm_id).all()

    if new_product.validate_on_submit():
        try:
            farm_product_catalog_id = max([value for value, in db.session.query(FarmProduct.farm_product_catalog_id).all()]) + 1
        except Exception as e:
            farm_product_catalog_id = 1

        farm_id = request.args['id']
        products = db.session.query(FarmProduct.product_id, ProductStore.product_description).join(ProductStore, ProductStore.product_id == FarmProduct.product_id).filter(FarmProduct.farm_id==farm_id).all()

        additional_product = FarmProduct(farm_product_catalog_id=farm_product_catalog_id,
                                    farm_id=farm_id,
                                    product_id=new_product.product.data)

        db.session.add(additional_product)
        db.session.commit()
        return redirect(url_for('home'))
    return render_template('add_product.html', farm=farm_id, products=products, form=new_product)
###
### The customer view table where we can track the various customers in the database
###
@app.route("/customer")




###
### The farm finance view where the farmers can track their finances
###
@app.route("/finances", methods=['GET', 'POST'])
def finances():
    finance_form = AddFinance()
    
    if request.method == "GET" or "POST":
        farm_id = request.args['id']
        income_transactions=db.session.query(Finance).filter(Finance.farm_id==farm_id).all()
        expense_transactions=db.session.query(Finance).filter(Finance.farm_id==farm_id).all()
        returns=db.session.query(Finance).filter(Finance.farm_id==farm_id).all()
        historical_dates=db.session.query(Finance).filter(Finance.farm_id==farm_id).all()
        expected_revenue = db.session.query(Finance).filter(Finance.farm_id==farm_id).all()
    if finance_form.validate_on_submit():

        try:
            tran_id = max([value for value, in db.session.query(Finance.tran_id).all()]) + 1
        except:
            tran_id = 1
        
        farm_id = request.args['id']
        income = finance_form.income.data
        expense = finance_form.expenses.data
        returns = finance_form.returns.data
        expected_revenue = income - expense- returns
        new_transaction = Finance(
                                    tran_id=tran_id,
                                    farm_id=farm_id,
                                    income=income,
                                    expense=expense,
                                    returns=returns,
                                    expected_revenue=expected_revenue
                                    )
        db.session.add(new_transaction)
        db.session.commit()
        return redirect(url_for('home'))

    return render_template('finances.html',
                                            farm_id=farm_id,
                                            income_transactions=income_transactions,
                                            expense_transactions=expense_transactions,
                                            historical_dates=historical_dates,
                                            returns=returns,
                                            expected_revenue=expected_revenue,
                                            form=finance_form)


###
### Strategic decisions
###

@app.route("/strategic_decisions", methods=['GET', 'POST'])
def strategic_decisions():

    render_template('strategic_decisions.html')


###
### Strategic decisions
###



    
###
### The mentor view where metnors can sign in, view farm states make comments/blog
###

@app.route("/")
def index():
    if current_user.is_authenticated:
        return redirect(url_for('home'))
    else:
        return '<a class ="btn btn-warning btn-lg" href="/login">Google Login</a>'


@app.route("/login")
def login():
    # Find out what URL to hit for Google login
    google_provider_cfg = get_google_provider_cfg()
    authorization_endpoint = google_provider_cfg["authorization_endpoint"]

    # Use library to construct the request for Google login and provide
    # scopes that let you retrieve user's profile from Google
    request_uri = client.prepare_request_uri(
        authorization_endpoint,
        redirect_uri=request.base_url + "/callback",
        scope=["openid", "email", "profile"],
    )
    return redirect(request_uri)


@app.route("/login/callback")
def callback():
    # Get authorization code Google sent back to you
    code = request.args.get("code")
    # Find out what URL to hit to get tokens that allow you to ask for
    # things on behalf of a user
    google_provider_cfg = get_google_provider_cfg()
    token_endpoint = google_provider_cfg["token_endpoint"]
    # Prepare and send a request to get tokens! Yay tokens!
    token_url, headers, body = client.prepare_token_request(
        token_endpoint,
        authorization_response=request.url,
        redirect_url=request.base_url,
        code=code
    )
    token_response = requests.post(
        token_url,
        headers=headers,
        data=body,
        auth=(GOOGLE_CLIENT_ID, GOOGLE_CLIENT_SECRET),
    )

    # Parse the tokens!
    client.parse_request_body_response(json.dumps(token_response.json()))
    # Now that you have tokens (yay) let's find and hit the URL
    # from Google that gives you the user's profile information,
    # including their Google profile image and email
    userinfo_endpoint = google_provider_cfg["userinfo_endpoint"]
    uri, headers, body = client.add_token(userinfo_endpoint)
    userinfo_response = requests.get(uri, headers=headers, data=body)

    # You want to make sure their email is verified.
    # The user authenticated with Google, authorized your
    # app, and now you've verified their email through Google!
    if userinfo_response.json().get("email_verified"):
        unique_id = userinfo_response.json()["sub"]
        users_email = userinfo_response.json()["email"]
        picture = userinfo_response.json()["picture"]
        users_name = userinfo_response.json()["given_name"]
    else:
        return "User email not available or not verified by Google.", 400

    # Create a user in your db with the information provided
    # by Google
    user = User(
        id=unique_id, name=users_name, email=users_email, profile_pic=picture
    )

    # Doesn't exist? Add it to the database.

    if not UserUtils.get(unique_id, db.session):
        UserUtils.create(unique_id, users_name, users_email, picture, db.session)

    # Begin user session by logging the user in
    login_user(user)

    # Send user back to homepage
    return redirect(url_for("index"))


@app.route("/logout")
#@login_required
def logout():
    logout_user()
    return redirect(url_for("index"))


if __name__ == "__main__":
    app.run(ssl_context='adhoc',debug=True, host='127.0.0.1', port=int(os.environ.get("PORT", 5000)))
